@@ -27,6 +27,7 @@
 #define KEY_ZOOM                  "zoom"
 #define KEY_ROTATE                "rotate"
 #define KEY_PAGES_PER_ROW         "pages-per-row"
+#define KEY_PAGE_RIGHT_TO_LEFT    "page-right-to-left"
 #define KEY_FIRST_PAGE_COLUMN     "first-page-column"
 #define KEY_POSITION_X            "position-x"
 #define KEY_POSITION_Y            "position-y"
@@ -37,19 +38,33 @@
 #include <sys/file.h>
 #define file_lock_set(fd, cmd) flock(fd, cmd)
 #else
-#define file_lock_set(fd, cmd) \
-  { \
-  struct flock lock = { .l_type = cmd, .l_start = 0, .l_whence = SEEK_SET, .l_len = 0}; \
-  fcntl(fd, F_SETLK, lock); \
-  }
+static int
+file_lock_set(int fd, short cmd)
+{
+  struct flock lock = { .l_type = cmd, .l_start = 0, .l_whence = SEEK_SET, .l_len = 0};
+  return fcntl(fd, F_SETLK, lock);
+}
 #endif
 
 static void zathura_database_interface_init(ZathuraDatabaseInterface* iface);
 static void io_interface_init(GiraraInputHistoryIOInterface* iface);
 
+typedef struct zathura_plaindatabase_private_s {
+  char* bookmark_path;
+  GKeyFile* bookmarks;
+  GFileMonitor* bookmark_monitor;
+
+  char* history_path;
+  GKeyFile* history;
+  GFileMonitor* history_monitor;
+
+  char* input_history_path;
+} ZathuraPlainDatabasePrivate;
+
 G_DEFINE_TYPE_WITH_CODE(ZathuraPlainDatabase, zathura_plaindatabase, G_TYPE_OBJECT,
                         G_IMPLEMENT_INTERFACE(ZATHURA_TYPE_DATABASE, zathura_database_interface_init)
-                        G_IMPLEMENT_INTERFACE(GIRARA_TYPE_INPUT_HISTORY_IO, io_interface_init))
+                        G_IMPLEMENT_INTERFACE(GIRARA_TYPE_INPUT_HISTORY_IO, io_interface_init)
+                        G_ADD_PRIVATE(ZathuraPlainDatabase))
 
 static void           plain_dispose(GObject* object);
 static void           plain_finalize(GObject* object);
@@ -71,21 +86,6 @@ static GKeyFile*      zathura_db_read_key_file_from_file(const char* path);
 static void           zathura_db_write_key_file_to_file(const char* file, GKeyFile* key_file);
 static void           cb_zathura_db_watch_file(GFileMonitor* monitor, GFile* file, GFile* other_file, GFileMonitorEvent event, zathura_database_t* database);
 
-typedef struct zathura_plaindatabase_private_s {
-  char* bookmark_path;
-  GKeyFile* bookmarks;
-  GFileMonitor* bookmark_monitor;
-
-  char* history_path;
-  GKeyFile* history;
-  GFileMonitor* history_monitor;
-
-  char* input_history_path;
-} zathura_plaindatabase_private_t;
-
-#define ZATHURA_PLAINDATABASE_GET_PRIVATE(obj) \
-  (G_TYPE_INSTANCE_GET_PRIVATE ((obj), ZATHURA_TYPE_PLAINDATABASE, zathura_plaindatabase_private_t))
-
 enum {
   PROP_0,
   PROP_PATH
@@ -130,9 +130,6 @@ io_interface_init(GiraraInputHistoryIOInterface* iface)
 static void
 zathura_plaindatabase_class_init(ZathuraPlainDatabaseClass* class)
 {
-  /* add private members */
-  g_type_class_add_private(class, sizeof(zathura_plaindatabase_private_t));
-
   /* override methods */
   GObjectClass* object_class = G_OBJECT_CLASS(class);
   object_class->dispose      = plain_dispose;
@@ -147,7 +144,7 @@ zathura_plaindatabase_class_init(ZathuraPlainDatabaseClass* class)
 static void
 zathura_plaindatabase_init(ZathuraPlainDatabase* db)
 {
-  zathura_plaindatabase_private_t* priv = ZATHURA_PLAINDATABASE_GET_PRIVATE(db);
+  ZathuraPlainDatabasePrivate* priv = zathura_plaindatabase_get_instance_private(db);
 
   priv->bookmark_path         = NULL;
   priv->bookmark_monitor      = NULL;
@@ -164,7 +161,7 @@ zathura_plaindatabase_new(const char* path)
   g_return_val_if_fail(path != NULL && strlen(path) != 0, NULL);
 
   zathura_database_t* db = g_object_new(ZATHURA_TYPE_PLAINDATABASE, "path", path, NULL);
-  zathura_plaindatabase_private_t* priv = ZATHURA_PLAINDATABASE_GET_PRIVATE(db);
+  ZathuraPlainDatabasePrivate* priv = zathura_plaindatabase_get_instance_private(ZATHURA_PLAINDATABASE(db));
   if (priv->bookmark_path == NULL) {
     g_object_unref(db);
     return NULL;
@@ -176,7 +173,7 @@ zathura_plaindatabase_new(const char* path)
 static void
 plain_db_init(ZathuraPlainDatabase* db, const char* dir)
 {
-  zathura_plaindatabase_private_t* priv = ZATHURA_PLAINDATABASE_GET_PRIVATE(db);
+  ZathuraPlainDatabasePrivate* priv = zathura_plaindatabase_get_instance_private(db);
 
   /* bookmarks */
   priv->bookmark_path = g_build_filename(dir, BOOKMARKS, NULL);
@@ -287,7 +284,7 @@ static void
 plain_dispose(GObject* object)
 {
   ZathuraPlainDatabase* db = ZATHURA_PLAINDATABASE(object);
-  zathura_plaindatabase_private_t* priv = ZATHURA_PLAINDATABASE_GET_PRIVATE(db);
+  ZathuraPlainDatabasePrivate* priv = zathura_plaindatabase_get_instance_private(db);
 
   g_clear_object(&priv->bookmark_monitor);
   g_clear_object(&priv->history_monitor);
@@ -299,7 +296,7 @@ static void
 plain_finalize(GObject* object)
 {
   ZathuraPlainDatabase* db = ZATHURA_PLAINDATABASE(object);
-  zathura_plaindatabase_private_t* priv = ZATHURA_PLAINDATABASE_GET_PRIVATE(db);
+  ZathuraPlainDatabasePrivate* priv = zathura_plaindatabase_get_instance_private(db);
 
   /* bookmarks */
   g_free(priv->bookmark_path);
@@ -325,7 +322,8 @@ static bool
 plain_add_bookmark(zathura_database_t* db, const char* file,
                    zathura_bookmark_t* bookmark)
 {
-  zathura_plaindatabase_private_t* priv = ZATHURA_PLAINDATABASE_GET_PRIVATE(db);
+  ZathuraPlainDatabase* plaindb     = ZATHURA_PLAINDATABASE(db);
+  ZathuraPlainDatabasePrivate* priv = zathura_plaindatabase_get_instance_private(plaindb);
   if (priv->bookmarks == NULL || priv->bookmark_path == NULL ||
       bookmark->id == NULL) {
     return false;
@@ -363,7 +361,8 @@ plain_add_bookmark(zathura_database_t* db, const char* file,
 static bool
 plain_remove_bookmark(zathura_database_t* db, const char* file, const char* id)
 {
-  zathura_plaindatabase_private_t* priv = ZATHURA_PLAINDATABASE_GET_PRIVATE(db);
+  ZathuraPlainDatabase* plaindb     = ZATHURA_PLAINDATABASE(db);
+  ZathuraPlainDatabasePrivate* priv = zathura_plaindatabase_get_instance_private(plaindb);
   if (priv->bookmarks == NULL || priv->bookmark_path == NULL) {
     return false;
   }
@@ -386,7 +385,8 @@ plain_remove_bookmark(zathura_database_t* db, const char* file, const char* id)
 static girara_list_t*
 plain_load_bookmarks(zathura_database_t* db, const char* file)
 {
-  zathura_plaindatabase_private_t* priv = ZATHURA_PLAINDATABASE_GET_PRIVATE(db);
+  ZathuraPlainDatabase* plaindb     = ZATHURA_PLAINDATABASE(db);
+  ZathuraPlainDatabasePrivate* priv = zathura_plaindatabase_get_instance_private(plaindb);
   if (priv->bookmarks == NULL) {
     return NULL;
   }
@@ -497,9 +497,10 @@ plain_load_jumplist(zathura_database_t* db, const char* file)
 {
   g_return_val_if_fail(db != NULL && file != NULL, NULL);
 
-  zathura_plaindatabase_private_t* priv = ZATHURA_PLAINDATABASE_GET_PRIVATE(db);
-  char* str_value = g_key_file_get_string(priv->history, file, KEY_JUMPLIST, NULL);
+  ZathuraPlainDatabase* plaindb     = ZATHURA_PLAINDATABASE(db);
+  ZathuraPlainDatabasePrivate* priv = zathura_plaindatabase_get_instance_private(plaindb);
 
+  char* str_value = g_key_file_get_string(priv->history, file, KEY_JUMPLIST, NULL);
   if (str_value == NULL) {
     return girara_list_new2(g_free);
   }
@@ -532,7 +533,8 @@ plain_save_jumplist(zathura_database_t* db, const char* file, girara_list_t* jum
   GString* str_val = g_string_new(NULL);
   girara_list_foreach(jumplist, jump_to_str, str_val);
 
-  zathura_plaindatabase_private_t* priv = ZATHURA_PLAINDATABASE_GET_PRIVATE(db);
+  ZathuraPlainDatabase* plaindb     = ZATHURA_PLAINDATABASE(db);
+  ZathuraPlainDatabasePrivate* priv = zathura_plaindatabase_get_instance_private(plaindb);
 
   g_key_file_set_string(priv->history, file, KEY_JUMPLIST, str_val->str);
   zathura_db_write_key_file_to_file(priv->history_path, priv->history);
@@ -545,7 +547,8 @@ static bool
 plain_set_fileinfo(zathura_database_t* db, const char* file, zathura_fileinfo_t*
                    file_info)
 {
-  zathura_plaindatabase_private_t* priv = ZATHURA_PLAINDATABASE_GET_PRIVATE(db);
+  ZathuraPlainDatabase* plaindb     = ZATHURA_PLAINDATABASE(db);
+  ZathuraPlainDatabasePrivate* priv = zathura_plaindatabase_get_instance_private(plaindb);
   if (priv->history == NULL || file_info == NULL || file == NULL) {
     return false;
   }
@@ -558,6 +561,7 @@ plain_set_fileinfo(zathura_database_t* db, const char* file, zathura_fileinfo_t*
   g_key_file_set_integer(priv->history, name, KEY_ROTATE,            file_info->rotation);
   g_key_file_set_integer(priv->history, name, KEY_PAGES_PER_ROW,     file_info->pages_per_row);
   g_key_file_set_string(priv->history, name, KEY_FIRST_PAGE_COLUMN,  file_info->first_page_column_list);
+  g_key_file_set_boolean(priv->history, name, KEY_PAGE_RIGHT_TO_LEFT,file_info->page_right_to_left);
   g_key_file_set_double (priv->history, name, KEY_POSITION_X,        file_info->position_x);
   g_key_file_set_double (priv->history, name, KEY_POSITION_Y,        file_info->position_y);
   g_key_file_set_integer(priv->history, name, KEY_TIME,              time(NULL));
@@ -577,7 +581,8 @@ plain_get_fileinfo(zathura_database_t* db, const char* file, zathura_fileinfo_t*
     return false;
   }
 
-  zathura_plaindatabase_private_t* priv = ZATHURA_PLAINDATABASE_GET_PRIVATE(db);
+  ZathuraPlainDatabase* plaindb     = ZATHURA_PLAINDATABASE(db);
+  ZathuraPlainDatabasePrivate* priv = zathura_plaindatabase_get_instance_private(plaindb);
   if (priv->history == NULL) {
     return false;
   }
@@ -600,6 +605,9 @@ plain_get_fileinfo(zathura_database_t* db, const char* file, zathura_fileinfo_t*
   if (g_key_file_has_key(priv->history, name, KEY_FIRST_PAGE_COLUMN, NULL) == TRUE) {
     file_info->first_page_column_list = g_key_file_get_string(priv->history, name, KEY_FIRST_PAGE_COLUMN, NULL);
   }
+  if (g_key_file_has_key(priv->history, name, KEY_PAGE_RIGHT_TO_LEFT, NULL) == TRUE) {
+    file_info->page_right_to_left = g_key_file_get_boolean(priv->history, name, KEY_PAGE_RIGHT_TO_LEFT, NULL);
+  }
   if (g_key_file_has_key(priv->history, name, KEY_POSITION_X, NULL) == TRUE) {
     file_info->position_x        = g_key_file_get_double(priv->history, name, KEY_POSITION_X, NULL);
   }
@@ -741,7 +749,8 @@ cb_zathura_db_watch_file(GFileMonitor* UNUSED(monitor), GFile* file, GFile* UNUS
     return;
   }
 
-  zathura_plaindatabase_private_t* priv = ZATHURA_PLAINDATABASE_GET_PRIVATE(database);
+  ZathuraPlainDatabase* plaindb     = ZATHURA_PLAINDATABASE(database);
+  ZathuraPlainDatabasePrivate* priv = zathura_plaindatabase_get_instance_private(plaindb);
   if (priv->bookmark_path && strcmp(priv->bookmark_path, path) == 0) {
     if (priv->bookmarks != NULL) {
       g_key_file_free(priv->bookmarks);
@@ -762,7 +771,8 @@ cb_zathura_db_watch_file(GFileMonitor* UNUSED(monitor), GFile* file, GFile* UNUS
 static girara_list_t*
 plain_io_read(GiraraInputHistoryIO* db)
 {
-  zathura_plaindatabase_private_t* priv = ZATHURA_PLAINDATABASE_GET_PRIVATE(db);
+  ZathuraPlainDatabase* plaindb     = ZATHURA_PLAINDATABASE(db);
+  ZathuraPlainDatabasePrivate* priv = zathura_plaindatabase_get_instance_private(plaindb);
 
   /* open file */
   FILE* file = fopen(priv->input_history_path, "r");
@@ -793,7 +803,8 @@ plain_io_read(GiraraInputHistoryIO* db)
 static void
 plain_io_append(GiraraInputHistoryIO* db, const char* input)
 {
-  zathura_plaindatabase_private_t* priv = ZATHURA_PLAINDATABASE_GET_PRIVATE(db);
+  ZathuraPlainDatabase* plaindb     = ZATHURA_PLAINDATABASE(db);
+  ZathuraPlainDatabasePrivate* priv = zathura_plaindatabase_get_instance_private(plaindb);
 
   /* open file */
   FILE* file = fopen(priv->input_history_path, "r+");
@@ -858,7 +869,8 @@ compare_time(const void* l, const void* r, void* data)
 static girara_list_t*
 plain_get_recent_files(zathura_database_t* db, int max, const char* basepath)
 {
-  zathura_plaindatabase_private_t* priv = ZATHURA_PLAINDATABASE_GET_PRIVATE(db);
+  ZathuraPlainDatabase* plaindb     = ZATHURA_PLAINDATABASE(db);
+  ZathuraPlainDatabasePrivate* priv = zathura_plaindatabase_get_instance_private(plaindb);
 
   girara_list_t* result = girara_list_new2(g_free);
   if (result == NULL) {
